@@ -3,18 +3,18 @@ from operator import itemgetter as at
 import numpy as np
 from pathlib import Path
 
-src = Path(__file__).absolute().parent
-sys.path.append(str(src))
-from encoders import PartitionSchema
+# src = Path(__file__).absolute().parent
+# sys.path.append(str(src))
+from .encoders import PartitionSchema
 from joblib import delayed, Parallel
-from similarity_helpers import parse_server_name, LazyHnsw
+from .similarity_helpers import parse_server_name, FlatFaiss
 
 class BaseStrategy:
     __slots__ = ["schema", "partitions","index_labels", "model_dir", "IndexEngine", "engine_params"]
     def __init__(self, model_dir=None, similarity_engine=None ,engine_params={}):
         if similarity_engine is None:
-            self.IndexEngine = LazyHnsw
-            self.engine_params = {"max_elements":10240,"ef_construction": 2000, "M": 16}
+            self.IndexEngine = FlatFaiss
+            self.engine_params = {}
         else:
             self.IndexEngine = parse_server_name(similarity_engine)
             self.engine_params = engine_params
@@ -29,7 +29,7 @@ class BaseStrategy:
 
     def init_schema(self, **kwargs):
         self.schema = PartitionSchema(**kwargs)
-        self.partitions = [self.IndexEngine(self.schema.metric, self.schema.dim, **self.engine_params) for _ in self.schema.partitions]
+        self.partitions = [self.IndexEngine(self.schema.metric, self.schema.dim, self.schema.index_factory, **self.engine_params) for _ in self.schema.partitions]
         enc_sizes = {k:len(v) for k,v in self.schema.encoders.items()}
         return self.schema.partitions, enc_sizes
 
@@ -39,7 +39,7 @@ class BaseStrategy:
         for datum in data:
             try:
                 vecs.append((self.schema.partition_num(datum), self.schema.encode(datum), datum[self.schema.id_col]))
-            except KeyError as e:
+            except Exception as e:
                 errors.append((datum, str(e)))
         vecs = sorted(vecs, key=at(0))
         affected_partitions = 0
@@ -56,6 +56,7 @@ class BaseStrategy:
             affected_partitions += 1
             num_ids = list(map(self.index_labels.index, ids))
             self.partitions[partition_num].add_items(items, num_ids)
+            self.schema.add_mapping(partition_num, num_ids, [d for d in data if d['id'] in ids])
         return errors, affected_partitions
 
 
@@ -100,19 +101,22 @@ class BaseStrategy:
         try:
             vec = vec.reshape(1,-1).astype('float32') # for faiss
             distances, num_ids = self.partitions[partition_num].search(vec, k=k)
+            indices = np.where(num_ids != -1)
+            distances, num_ids = distances[indices], num_ids[indices]
         except Exception as e:
             raise Exception("Error in querying: " + str(e))
         if len(num_ids) == 0:
             labels, distances = [], []
         else:
-            labels = [self.index_labels[n] for n in num_ids[0]]
-            distances = [float(d) for d in distances[0]]
+            labels = [self.index_labels[n] for n in num_ids]
+            distances = [float(d) for d in distances]
         if not explain:
             return labels,distances, []
 
         vec = vec.reshape(-1)
         explanation = []
-        X = self.partitions[partition_num].get_items(num_ids[0])
+        # X = self.partitions[partition_num].get_items(num_ids[0])
+        X = np.array([self.schema.restore_vector_with_index(partition_num, index) for index in num_ids[0]], dtype='float32')
         first_sim = None
         for ret_vec in X:
             start=0
@@ -142,14 +146,16 @@ class BaseStrategy:
     def query(self, data, k, explain=False):
         try:
             partition_nums = self.schema.partition_num(data)
+            if type(partition_nums) != list:
+                partition_nums = [partition_nums]
         except Exception as e:
             raise Exception("Error in partitioning: " + str(e))
         try:
             vec = self.schema.encode(data)
         except Exception as e:
             raise Exception("Error in encoding: " + str(e))
-        if type(partition_nums)!=list:
             return self.query_by_partition_and_vector(partition_nums, vec, k, explain)
+
         # Aggregate results if multiple partitions are returned:
         labels,distances,explanation = [], [], []
         for partition_num in partition_nums:
@@ -209,16 +215,21 @@ class BaseStrategy:
 
     def fetch(self, lbls, numpy=False):
         sil = set(self.index_labels)
-        found = [l for l in lbls if l in sil]
+        if type(lbls) == list:
+            found = [l for l in lbls if l in sil]
+        else:
+            found = [lbls] if lbls in sil else []
         ids = [self.index_labels.index(l) for l in found]
         ret = collections.defaultdict(list)
-        for p,pn in zip(self.partitions, self.schema.partitions):
+        for partition_num, (p,pn) in enumerate(zip(self.partitions, self.schema.partitions)):
             for id in ids:
                 try:
                     if numpy:
-                        ret[pn].extend(p.get_items([id]))
+                        # ret[pn].extend(p.get_items([id]))
+                        ret[pn].extend(self.schema.restore_vector_with_index(partition_num, id))
                     else:
-                        ret[pn].extend([tuple(float(v) for v in vec) for vec in p.get_items([id])])
+                        # ret[pn].extend([tuple(float(v) for v in vec) for vec in p.get_items([id])])
+                        ret[pn].extend([tuple(float(v) for v in vec) for vec in self.schema.restore_vector_with_index(partition_num, id)])
                 except Exception as e:
                     # not found
                     pass
@@ -253,23 +264,25 @@ class AvgUserStrategy(BaseStrategy):
         # Assumes same features as the item
         return self.schema.partition_num(user_data)
 
-    def user_query(self, user_data, item_history, k, user_coldstart_item=None):
+    def user_query(self, user_data, item_history, k, user_coldstart_item=None,user_coldstart_weight=1):
         if user_coldstart_item is None:
             n = 0
             vec = np.zeros(self.schema.dim)
         else:
-            n = 1
+            n = user_coldstart_weight
             vec = self.schema.encode(user_coldstart_item)
         user_partition_num = self.user_partition_num(user_data)
         col_mapping = self.schema.component_breakdown()
         labels,distances = [], []
         if type(item_history) == str:
-            item_history = [item_history]
-        for item in item_history:
+            item_history = {item_history:1}
+        elif type(item_history) == list:
+            item_history = {i:1 for i in item_history}
+        for item, w in item_history.items():
             # Calculate AVG
             item_vectors = [v for vs in self.fetch(item, numpy=True).values() for v in vs]
-            n+=len(item_vectors)
-            vec += np.sum(item_vectors, axis=0)
+            n+=w*len(item_vectors)
+            vec += w*np.sum(item_vectors, axis=0)
         if n>0:
             vec /= n
         # Override column values post aggregation, if needed
